import { Component, inject, OnInit } from '@angular/core';
import { ActionListComponent } from "../action-list/action-list.component";
import { MatSelectModule } from '@angular/material/select';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';
import { MatIconModule } from '@angular/material/icon';
import { HermesClientService } from '../../hermes-client.service';
import RedeemableAction from '../../shared/models/redeemable_action';
import { FormControl, ReactiveFormsModule } from '@angular/forms';

interface IActionFilter {
  name: string
  filter: (action: any) => boolean
}

@Component({
  selector: 'actions',
  standalone: true,
  imports: [
    ActionListComponent,
    ReactiveFormsModule,
    MatFormFieldModule,
    MatIconModule,
    MatInputModule,
    MatSelectModule
  ],
  templateUrl: './actions.component.html',
  styleUrl: './actions.component.scss'
})
export class ActionsComponent implements OnInit {
  filters: IActionFilter[] = [
    { name: 'All', filter: _ => true },
    { name: 'Local File', filter: data => data.type.includes('_FILE') },
    { name: 'Nightbot', filter: data => data.type.includes('NIGHTBOT_') },
    { name: 'OBS', filter: data => data.type.includes('OBS_') },
    { name: 'Sleep', filter: data => data.type == "SLEEP" },
    { name: 'TTS', filter: data => data.type.includes('TTS') },
    { name: 'Veadotube', filter: data => data.type.includes('VEADOTUBE') },
  ];

  client = inject(HermesClientService);
  filter = this.filters[0];
  searchControl = new FormControl('');
  search = '';
  items: RedeemableAction[] = [];

  ngOnInit(): void {
    this.client.subscribeToRequests('get_redeemable_actions', d => {
      this.items = d.data;
    });
    this.client.subscribeToRequests('create_redeemable_action', d => {
      if (d.request.nounce != null && d.request.nounce.startsWith(this.client.session_id)) {
        return;
      }

      this.actions.push(d.data);
    });
    this.client.subscribeToRequests('update_redeemable_action', d => {
      if (d.request.nounce != null && d.request.nounce.startsWith(this.client.session_id)) {
        return;
      }

      const action = this.actions.find(a => a.name == d.data.name);
      if (action) {
        action.type = d.data.type;
        action.data = d.data.data;
      }
    });
    this.client.subscribeToRequests('delete_redeemable_action', d => {
      // if (d.request.nounce != null && d.request.nounce.startsWith(this.client.session_id)) {
      //   return;
      // }

      this.items = this.actions.filter(a => a.name != d.request.data.name);
    });

    this.client.fetchRedeemableActions();
  }

  get actions(): RedeemableAction[] {
    const searchLower = this.search.toLowerCase();
    return this.items.filter(this.filter.filter)
      .filter((action) => action.name.toLowerCase().includes(searchLower));
  }

  set actions(value) {
    this.items = value;
  }

  onFilterChange(event: any): void {
    this.filter = this.filters[event];
  }
}
