import { Component, inject, model } from '@angular/core';
import { MatDialogRef, MAT_DIALOG_DATA, MatDialogActions, MatDialogClose, MatDialogTitle, MatDialogContent } from '@angular/material/dialog';
import { Filter } from '../../shared/models/filter';
import { FormControl, FormGroup, FormsModule, ReactiveFormsModule, Validators } from '@angular/forms';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';
import { MatButtonModule } from '@angular/material/button';

@Component({
  selector: 'tts-filter-item-edit',
  standalone: true,
  imports: [
    FormsModule,
    MatButtonModule,
    MatDialogActions,
    MatDialogClose,
    MatDialogContent,
    MatDialogTitle,
    MatFormFieldModule,
    MatInputModule,
    ReactiveFormsModule,
  ],
  templateUrl: './filter-item-edit.component.html',
  styleUrl: './filter-item-edit.component.scss'
})
export class FilterItemEditComponent {
  readonly dialogRef = inject(MatDialogRef<FilterItemEditComponent>);
  readonly data = inject<Filter>(MAT_DIALOG_DATA);
  readonly forms = new FormGroup({
    search: new FormControl(this.data.search, [Validators.required]),
    replace: new FormControl(this.data.replace),
    flag: new FormControl(this.data.flag),
  });


  onSaveClick(): Filter|undefined {
    if (this.forms.invalid) {
      return undefined;
    }

    this.data.search = this.forms.value.search ?? '';
    this.data.replace = this.forms.value.replace ?? '';
    this.data.flag = this.forms.value.flag ?? 0;
    return this.data;
  }

  onCancelClick(): void {
    this.dialogRef.close();
  }
}
