import { Component, inject, Inject, Input, OnDestroy, OnInit, PLATFORM_ID, signal } from '@angular/core';
import { FilterListComponent } from "../filter-list/filter-list.component";
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { HermesClientService } from '../../hermes-client.service';
import { Filter } from '../../shared/models/filter';
import { isPlatformBrowser } from '@angular/common';
import { Router } from '@angular/router';
import { FilterItemEditComponent } from '../filter-item-edit/filter-item-edit.component';
import { MatDialog } from '@angular/material/dialog';

@Component({
    selector: 'filters',
    standalone: true,
    imports: [FilterListComponent, MatButtonModule, MatIconModule],
    templateUrl: './filters.component.html',
    styleUrl: './filters.component.scss'
})
export class FiltersComponent implements OnInit, OnDestroy {
  private isBrowser: boolean;
  readonly dialog = inject(MatDialog);
  items: Filter[];


  constructor(private client: HermesClientService, private router: Router, @Inject(PLATFORM_ID) private platformId: Object) {
    this.isBrowser = isPlatformBrowser(this.platformId);

    this.items = []
    this.client.subscribe(4, d => {
      const type = d.request.type;

      if (type == 'get_tts_word_filters') {
        this.items = d.data;
        return;
      }
      if (d.request.nounce == client.session_id) {
        return;
      }
      if (type == 'create_tts_filter') {
        this.items = [d.data, ...this.items];
      } else if (type == 'delete_tts_filter') {
        this.items = this.items.filter(i => i.id != d.data.id);
      } else if (type == 'update_tts_filter') {
        const filter = this.items.find(f => f.id == d.data.id);
        if (filter == null)
          return;

        filter.search = d.data.search;
        filter.replace = d.data.replace;
        filter.flag = d.data.flag || 0;
      }
    });
    this.client.fetchFilters();
  }

  ngOnInit(): void {
    if (!this.isBrowser)
      return;

    if (!this.client.logged_in) {
      this.router.navigate(["/tts-login"]);
      return;
    }
  }

  ngOnDestroy() {
  }

  openDialog(): void {
    const dialogRef = this.dialog.open(FilterItemEditComponent, {
      data: { search: '', replace: '' },
    });

    dialogRef.afterClosed().subscribe((result: any) => {
      if (result !== undefined) {
        this.client.first(d => d.op == 4 && d.d.request.type == 'create_tts_filter' && d.d.data.search == result.search && d.d.data.replace == result.replace)
          ?.subscribe(d => {
            this.items = [d.d.data, ...this.items];
          });
        this.client.createTTSFilter(result.search, result.replace);
      }
    });
  }
}